package de.fhdw.wtf.dsl.scanner.test;

import de.fhdw.wtf.common.stream.TokenStream;
import de.fhdw.wtf.common.token.Token;

/**
 * TokenStream-Implementation for test cases. Prints detailed information about token, when fetched from this stream to
 * the command line.
 * 
 */
public class VerboseTokenStream implements TokenStream {
	
	/**
	 * Wrapped TokenStream that implements all features.
	 */
	private final TokenStream stream;
	
	/**
	 * Creates a verbose token stream, that adds detailed log information when accessing the wrapped stream.
	 * 
	 * @param wrappedStream
	 *            stream that implements necessary operations
	 */
	public VerboseTokenStream(final TokenStream wrappedStream) {
		this.stream = wrappedStream;
	}
	
	@Override
	public void add(final Token t) {
		this.getStream().add(t);
		System.out.println("Added -> " + t.toString());
	}
	
	@Override
	public Token next() {
		final Token next = this.getStream().next();
		System.out.println("Next -> " + next.toString());
		this.assertEndTokenIsLastToken(next);
		return next;
	}
	
	/**
	 * Asserts that (a possible) EndToken is the last Token of this stream.
	 * 
	 * @param next
	 *            token, which is tested for EndToken
	 * @throws RuntimeException
	 *             if this rule is violated. Definitely denotes a bug - don't try catching it!
	 */
	private void assertEndTokenIsLastToken(final Token next) {
		if (next.isEndToken() && this.getStream().hasNext()) {
			throw new RuntimeException(
					"After an EndToken NEVER is a next token. This exception shows a violation of this rule.");
		}
	}
	
	@Override
	public boolean hasNext() {
		return this.getStream().hasNext();
	}
	
	/**
	 * @return Returns the wrapped stream.
	 */
	public TokenStream getStream() {
		return this.stream;
	}
	
	@Override
	public boolean removeFirst() {
		return this.stream.removeFirst();
	}
	
	@Override
	public Token peek() {
		final Token t = this.getStream().peek();
		System.out.println("Peek --> " + t.toString());
		return t;
	}
	
	public void printStreamContent() {
		while (this.hasNext()) {
			System.out.print(this.getStream().next().stringRepresentation());
		}
	}
	
	@Override
	public TokenStream copy() {
		return new VerboseTokenStream(this.stream.copy());
	}
}
